package com.kb.db;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.LogicalExpression;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;

import com.kb.model.Employee;
import com.kb.util.HibernateUtil;

public class CriteriaExample {
	@SuppressWarnings("deprecation")
	public static void main(String[] args) {
		// Get session factory using Hibernate Util class
		SessionFactory sf = HibernateUtil.getSessionFactory();
		// Get session from Sesson factory
		Session session = sf.openSession();

		Criteria criteria = session.createCriteria(Employee.class);

		// Retrieve All the Employees using Criteria
		System.out.println("All the empoyees retrieved using Criteria");
		displayEmployeeDetails(criteria.list());

		criteria = session.createCriteria(Employee.class);
		Criterion employeeIdCriterion = Restrictions.eq("employeeId", 2);
		criteria.add(employeeIdCriterion);
		System.out.println("Employee record whose id is 2");
		displayEmployeeDetails((Employee) criteria.uniqueResult());

		criteria = session.createCriteria(Employee.class);
		Criterion greaterThanCriterion = Restrictions.gt("salary", 25000);
		criteria.add(greaterThanCriterion);
		System.out.println("All the empoyees whose salary is greater than 25000");
		displayEmployeeDetails(criteria.list());

		criteria = session.createCriteria(Employee.class);
		Criterion greaterThanOrEqualCriterion = Restrictions.ge("salary", 25000);
		criteria.add(greaterThanOrEqualCriterion);
		System.out.println("All the empoyees whose salary is greater than or equal to 25000");
		displayEmployeeDetails(criteria.list());

		criteria = session.createCriteria(Employee.class);
		Criterion lessThanCriterion = Restrictions.lt("salary", 25000);
		criteria.add(lessThanCriterion);
		System.out.println("All the empoyees whose salary is less than 25000");
		displayEmployeeDetails(criteria.list());

		criteria = session.createCriteria(Employee.class);
		Criterion lessThanOrEqualCriterion = Restrictions.le("salary", 25000);
		criteria.add(lessThanOrEqualCriterion);
		System.out.println("All the empoyees whose salary is less than or equal to 25000");
		displayEmployeeDetails(criteria.list());

		// "Like" example
		List<Employee> empList = session.createCriteria(Employee.class).add(Restrictions.like("firstName", "%o%"))
				.list();
		System.out.println("All the empoyees whose first name has a letter 'o'");
		displayEmployeeDetails(empList);

		// Pagination list example
		empList = session.createCriteria(Employee.class).addOrder(Order.desc("salary")).setFirstResult(0)
				.setMaxResults(2).list();
		System.out.println("Pgination result with max 2 records per page");
		displayEmployeeDetails(empList);

		// Combining multiple restrictions
		criteria = session.createCriteria(Employee.class);
		Criterion salaryGreaterThan = Restrictions.gt("salary", 25000);
		Criterion ageGreaterThanOrEqual = Restrictions.ge("age", 28);
		Criterion education = Restrictions.in("education", "PG", "Phd");
		LogicalExpression orExp = Restrictions.or(salaryGreaterThan, ageGreaterThanOrEqual);
		criteria.add(orExp);
		criteria.add(education);
		System.out.println(
				"All the empoyees whose salary is greater than 25000 or whose age is greater than 28 and educaton is either PG or Phd");
		displayEmployeeDetails(criteria.list());
		session.close();
	}

	private static void displayEmployeeDetails(List<Employee> employeeList) {
		for (Employee employee : employeeList) {
			displayEmployeeDetails(employee);
		}
	}

	private static void displayEmployeeDetails(Employee employee) {
		System.out.println(
				"ID: " + employee.getEmployeeId() + " Age: " + employee.getAge() + " Salary: " + employee.getSalary());
	}

}
